@@ -16,6 +16,8 @@ import functools
 
 import apt
 
+from UbuntuDrivers import kerneldetection
+
 system_architecture = apt.apt_pkg.get_architectures()[0]
 
 def system_modaliases():
@@ -28,9 +30,7 @@ def system_modaliases():
     suitable for a PackageKit WhatProvides(MODALIAS) call.
     '''
     aliases = {}
-    # $SYSFS_PATH is compatible with libudev
-    sysfs_dir = os.environ.get('SYSFS_PATH', '/sys')
-    for path, dirs, files in os.walk(os.path.join(sysfs_dir, 'devices')):
+    for path, dirs, files in os.walk('/sys/devices'):
         modalias = None
 
         # most devices have modalias files
@@ -39,7 +39,7 @@ def system_modaliases():
                 with open(os.path.join(path, 'modalias')) as f:
                     modalias = f.read().strip()
             except IOError as e:
-                logging.warning('system_modaliases(): Cannot read %s/modalias: %s',
+                logging.debug('system_modaliases(): Cannot read %s/modalias: %s',
                         path, e)
                 continue
 
@@ -127,7 +127,7 @@ def _apt_cache_modalias_map(apt_cache):
         # skip packages without a modalias field
         try:
             m = package.candidate.record['Modaliases']
-        except (KeyError, AttributeError):
+        except (KeyError, AttributeError, UnicodeDecodeError):
             continue
 
         # skip incompatible video drivers
@@ -244,32 +244,22 @@ def _get_db_name(syspath, alias):
 
     Values are None if unknown.
     '''
-    # ensure syspath is a device name relative to the sysfs dir
-    syspath = syspath[syspath.index('/devices/'):]
-
-    # check if we have an udev helper for this
-    command = '/lib/udev/%s-db' % alias.split(':')[0]
-    if not os.path.exists(command):
-        logging.debug('_get_db_name(%s, %s): No program %s, cannot identify',
-                      syspath, alias, command)
+    try:
+        out = subprocess.check_output(['udevadm', 'hwdb', '--test=' + alias],
+                                      universal_newlines=True)
+    except (OSError, subprocess.CalledProcessError) as e:
+        logging.debug('_get_db_name(%s, %s): udevadm hwdb failed: %s', syspath, alias, str(e))
         return (None, None)
 
-    # call udev ID helper
-    udev_db = subprocess.Popen([command, syspath], stdout=subprocess.PIPE,
-                               stderr = subprocess.PIPE, universal_newlines=True)
-    (out, err) = udev_db.communicate()
-    if udev_db.returncode != 0:
-        logging.debug('_get_db_name(%s, %s): %s failed with %i:\n%s', syspath,
-                      alias, command, udev_db.returncode, err)
-        return (None, None)
+    logging.debug('_get_db_name: output\n%s\n', out)
 
     vendor = None
     model = None
     for line in out.splitlines():
         (k, v) = line.split('=', 1)
-        if k == 'ID_VENDOR_FROM_DATABASE':
+        if '_VENDOR' in k:
             vendor = v
-        if k == 'ID_MODEL_FROM_DATABASE':
+        if '_MODEL' in k:
             model = v
 
     logging.debug('_get_db_name(%s, %s): vendor "%s", model "%s"', syspath,
@@ -561,3 +551,12 @@ def _add_builtins(drivers):
                     'free': True, 'builtin': True, 'from_distro': True, 'recommended': True}
                 break
 
+def get_linux_headers(apt_cache):
+    '''Return the linux headers for the system's kernel'''
+    kernel_detection = kerneldetection.KernelDetection(apt_cache)
+    return kernel_detection.get_linux_headers_metapackage()
+
+def get_linux(apt_cache):
+    '''Return the linux metapackage for the system's kernel'''
+    kernel_detection = kerneldetection.KernelDetection(apt_cache)
+    return kernel_detection.get_linux_metapackage()
